/* IO module for i8255 PIO for PC-Share V3.x
	Rename from IO_PORTS.CPP */

#ifdef _Windows
	#include <windows.h>
#endif
#ifdef __WIN32__
	#include "portio32.h"
#else
	#include <conio.h>
#endif

#include "globals.h"
#include "io_ports.h"

#define PORT_A 0
#define PORT_B 1
#define PORT_C 2
#define CTRL_REG 3

#define INPUT 0
#define OUTPUT 1

// current direction
int current_dir, base_addr ;

// internals
void init_out_port ( void ) ;
void init_in_port ( void ) ;

#ifdef _Windows

/* the DLL stuff for Windows build */

#ifdef __WIN32__

/* 32-bit version */
#pragma argsused
BOOL WINAPI DllEntryPoint(HINSTANCE hDLLInst, DWORD  fdwReason,
					LPVOID lpvReserved)
{
	return TRUE ;
}

#else

/* 16-bit version */

// Turn off warning: Parameter '' is never used
#pragma argsused
int FAR PASCAL LibMain( HINSTANCE hInstance,
								WORD wDataSegment,
								WORD wHeapSize,
								LPSTR lpszCmdLine )
{

// The startup code for the DLL initializes the local heap(if there is one)
// with a call to LocalInit which locks the data segment.

	 if ( wHeapSize != 0 )
		  UnlockData( 0 );
	 return 1;   // Indicate that the DLL was initialized successfully.
}

// Turn off warning: Parameter '' is never used
#pragma argsused
int FAR PASCAL WEP ( int bSystemExit )
{
	 return 1;
}

/* end of win16/32 specific */
#endif

/* end of Win specific */
#endif

// set base addr of port to be used
#pragma argsused
void far __pascal _export set_port_base ( int port_base, int irq, int dma )
{
	base_addr = port_base ;
	// initialise port
	init_in_port() ;
}

// get in_port status
int far __pascal _export in_port_status ( void )
{
	if ( current_dir != INPUT )
		init_in_port() ;

	if ( ( inp ( ( base_addr + PORT_C ) ) & 2 ) == 2 )
		return TRUE ;
	else
		return FALSE ;
}

// read a data byte from port
unsigned char far __pascal _export read_data ( void )
{
	if ( current_dir != INPUT )
		init_in_port() ;

	while ( ! in_port_status() )
	{
	}

	return inp ( base_addr + PORT_B ) ;
}

// initialise 8255 for input
void init_in_port ( void )
{
	int mode_1b_input = 134 ;

	mode_1b_input = outp ( ( base_addr + CTRL_REG ), mode_1b_input ) ;
	current_dir = INPUT ;
}

// initialise 8255 for output
void init_out_port ( void )
{
	int mode_1b_output = 132 ;

	mode_1b_output = outp ( base_addr + CTRL_REG, mode_1b_output ) ;
	current_dir = OUTPUT ;
}

// write data byte to outport port
void far __pascal _export write_data ( unsigned char data_byte )
{
	if ( current_dir != OUTPUT )
		init_out_port() ;

	outp ( base_addr + PORT_B, data_byte ) ;

	while ( !out_port_status())
	{
	}
}

// get status of output port
int far __pascal _export out_port_status ( void )
{
	if ( current_dir != OUTPUT )
		init_out_port() ;

	if ( ( inp ( base_addr + PORT_C ) & 2 ) == 2 )
		return TRUE ;
	else
		return FALSE ;
}

#ifndef _WINNT
// does nothing for all but formal NT build
void far __pascal _export close_io_port ( void )
{
}
#endif

